@@ -17,6 +17,7 @@ from flexlate.finder.specific.copier import CopierFinder
 from flexlate.template.base import Template
 from flexlate.template.types import TemplateType
 from flexlate.template_data import TemplateData
+from flexlate.template_path import get_local_repo_path_and_name_cloning_if_repo_url
 
 
 class TemplateSource(BaseModel):
@@ -26,6 +27,8 @@ class TemplateSource(BaseModel):
     version: Optional[str] = None
     git_url: Optional[str] = None
     target_version: Optional[str] = None
+    render_relative_root_in_output: Path = Path(".")
+    render_relative_root_in_template: Path = Path(".")
 
     _config_file_location: Path = PrivateAttr()
 
@@ -35,14 +38,18 @@ class TemplateSource(BaseModel):
     ) -> "TemplateSource":
         return cls(
             name=template.name,
-            path=str(template.path),
+            path=template.git_url
+            if template.git_url is not None
+            else str(template.path),
             version=template.version,
             type=template._type,
             target_version=target_version,
             git_url=template.git_url,
+            render_relative_root_in_output=template.render_relative_root_in_output,
+            render_relative_root_in_template=template.render_relative_root_in_template,
         )
 
-    def to_template(self) -> Template:
+    def to_template(self, version: Optional[str] = None) -> Template:
         if self.type == TemplateType.BASE:
             raise InvalidTemplateTypeException(
                 "base type is not allowed for concrete templates"
@@ -57,13 +64,26 @@ class TemplateSource(BaseModel):
                 f"no handling for template type {self.type} in creating template from source"
             )
         kwargs = dict(name=self.name)
-        if self.version is not None:
-            kwargs.update(version=self.version)
+        version = version or self.version
+        if version is not None:
+            kwargs.update(version=version)
+        # TODO: Can we remove target_version from templates?
         if self.target_version is not None:
             kwargs.update(target_version=self.target_version)
         if self.git_url is not None:
             kwargs.update(git_url=self.git_url)
-        local_path = self.absolute_local_path
+        local_path: Path
+        if self.git_url is not None:
+            # TODO: Avoid unnecessary git repo cloning
+            #  We already know that we have it by this point, but need to get the local path
+            #  and the logic to resolve the version that may be None is entertwined with the
+            #  cloning and local path determination
+            local_path, _ = get_local_repo_path_and_name_cloning_if_repo_url(
+                self.git_url, version=version
+            )
+        else:
+            local_path = self.absolute_local_path
+
         template = finder.find(self.git_url or str(local_path), local_path, **kwargs)
         # Keep original template source path (may be relative), so that later when
         # updating templates, it can update the path without forcing it to be absolute
@@ -91,15 +111,28 @@ class AppliedTemplateConfig(BaseModel):
     name: str
     data: TemplateData
     version: str
+    add_mode: AddMode
     root: Path = Path(".")
 
+    _config_file_location: Path = PrivateAttr()
+    _orig_root: Path = PrivateAttr()
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        # Root may get updated externally during config loading when adjusting applied paths,
+        # save the original root
+        self._orig_root = self.root
+
 
 class AppliedTemplateWithSource(BaseModel):
     applied_template: AppliedTemplateConfig
     source: TemplateSource
 
     def to_template_and_data(self) -> Tuple[Template, TemplateData]:
-        return self.source.to_template(), self.applied_template.data
+        return (
+            self.source.to_template(version=self.applied_template.version),
+            self.applied_template.data,
+        )
 
 
 class TemplateSourceWithTemplates(BaseModel):
@@ -173,6 +206,11 @@ class FlexlateConfig(BaseConfig):
             # can be made absolute before usage
             loaded_path = path or cls._settings.config_location
             template_source._config_file_location = Path(loaded_path)
+        for applied_template in config.applied_templates:
+            # Add location from which config was loaded so that it is easier to move
+            # applied templates to correct location after updates
+            loaded_path = path or cls._settings.config_location
+            applied_template._config_file_location = Path(loaded_path)
         return config
 
     def save(self, serializer_kwargs: Optional[Dict[str, Any]] = None, **kwargs):
