@@ -27,6 +27,7 @@ from flexlate.path_ops import (
     location_relative_to_new_parent,
     make_absolute_path_from_possibly_relative_to_another_path,
 )
+from flexlate.render.multi import MultiRenderer
 from flexlate.render.renderable import Renderable
 from flexlate.template.base import Template
 from flexlate.template_data import TemplateData, merge_data
@@ -119,25 +120,19 @@ class ConfigManager:
         self, project_root: Path = Path("."), config: Optional[FlexlateConfig] = None
     ) -> List[Renderable]:
         config = config or self.load_config(project_root)
-        renderables: List[Renderable] = []
-        for applied_with_source in self.get_applied_templates_with_sources(
-            project_root=project_root, config=config
-        ):
-            template, data = applied_with_source.to_template_and_data()
-            renderable: Renderable[Template] = Renderable(
-                template=template,
-                data=data,
-                out_root=applied_with_source.applied_template.root,
+        return [
+            Renderable.from_applied_template_with_source(applied_with_source)
+            for applied_with_source in self.get_applied_templates_with_sources(
+                project_root=project_root, config=config
             )
-            renderables.append(renderable)
-        return renderables
+        ]
 
     def get_renderables_for_updates(
         self,
         updates: Sequence[TemplateUpdate],
         project_root: Path = Path("."),
         adjust_root: bool = True,
-    ):
+    ) -> List[Renderable]:
         return [
             update.to_renderable(project_root=project_root, adjust_root=adjust_root)
             for update in updates
@@ -167,7 +162,7 @@ class ConfigManager:
         for child_config in config.child_configs:
             for i, applied_template in enumerate(child_config.applied_templates):
                 source = sources[applied_template.name]
-                template = source.to_template()
+                template = source.to_template(version=applied_template.version)
                 template.version = applied_template.version
                 updates.append(
                     TemplateUpdate(
@@ -206,7 +201,7 @@ class ConfigManager:
             # detected absolute location in the template.
             template_source.path = (
                 str(update.template.template_source_path)
-                if use_template_source_path and template_source.is_local_template
+                if use_template_source_path
                 else str(update.template.path)
             )
         self.save_config(config)
@@ -218,7 +213,7 @@ class ConfigManager:
         target_version: Optional[str] = None,
         project_root: Path = Path("."),
     ):
-        config = self.load_config(project_root=project_root)
+        config = self.load_config(project_root=project_root, adjust_applied_paths=False)
         child_config = _get_or_create_child_config_by_path(config, config_path)
         source = TemplateSource.from_template(
             template,
@@ -233,7 +228,7 @@ class ConfigManager:
         config_path: Path,
         project_root: Path = Path("."),
     ):
-        config = self.load_config(project_root=project_root)
+        config = self.load_config(project_root=project_root, adjust_applied_paths=False)
         if self._applied_template_exists_in_project(
             template_name, project_root=project_root, config=config
         ):
@@ -274,17 +269,19 @@ class ConfigManager:
         self,
         template: Template,
         config_path: Path,
+        add_mode: AddMode,
         data: Optional[TemplateData] = None,
         project_root: Path = Path("."),
         out_root: Path = Path("."),
     ):
-        config = self.load_config(project_root=project_root)
+        config = self.load_config(project_root=project_root, adjust_applied_paths=False)
         child_config = _get_or_create_child_config_by_path(config, config_path)
         applied = AppliedTemplateConfig(
             name=template.name,
             data=data or {},
             version=template.version,
             root=out_root,
+            add_mode=add_mode,
         )
         child_config.applied_templates.append(applied)
         self.save_config(config)
@@ -307,7 +304,42 @@ class ConfigManager:
             when working in a temp directory)
         :return:
         """
-        config = self.load_config(project_root=project_root)
+        template_index, _ = self._find_applied_template(
+            template_name,
+            config_path,
+            project_root=project_root,
+            out_root=out_root,
+            orig_project_root=orig_project_root,
+            adjust_applied_paths=True,
+        )
+        config = self.load_config(project_root=project_root, adjust_applied_paths=False)
+        child_config = _get_or_create_child_config_by_path(config, config_path)
+        child_config.applied_templates.pop(template_index)
+        self.save_config(config)
+
+    def _find_applied_template(
+        self,
+        template_name: str,
+        config_path: Path,
+        project_root: Path = Path("."),
+        out_root: Path = Path("."),
+        orig_project_root: Path = Path("."),
+        config: Optional[FlexlateConfig] = None,
+        adjust_applied_paths: bool = True,
+    ) -> Tuple[int, AppliedTemplateConfig]:
+        """
+
+        :param template_name:
+        :param config_path:
+        :param project_root: The root of the current working project (may be a temp directory)
+        :param out_root:
+        :param orig_project_root: The root of the user's project (always stays the same, even
+            when working in a temp directory)
+        :return: index, applied template config tuple
+        """
+        config = config or self.load_config(
+            project_root=project_root, adjust_applied_paths=adjust_applied_paths
+        )
         child_config = _get_or_create_child_config_by_path(config, config_path)
         template_index: Optional[int] = None
         orig_config_folder = location_relative_to_new_parent(
@@ -318,10 +350,14 @@ class ConfigManager:
         absolute_out_root = make_absolute_path_from_possibly_relative_to_another_path(
             out_root, orig_config_folder
         )
+        applied_template_reference_path = (
+            orig_project_root if adjust_applied_paths else orig_config_folder
+        )
         for i, applied_template in enumerate(child_config.applied_templates):
+
             absolute_template_out_root = (
                 make_absolute_path_from_possibly_relative_to_another_path(
-                    applied_template.root, orig_project_root
+                    applied_template.root, applied_template_reference_path
                 )
             )
             if (
@@ -334,8 +370,39 @@ class ConfigManager:
             raise CannotRemoveAppliedTemplateException(
                 f"Cannot find any applied template with name {template_name} and root {out_root}"
             )
-        child_config.applied_templates.pop(template_index)
-        self.save_config(config)
+        applied_template = child_config.applied_templates[template_index]
+        return template_index, applied_template
+
+    def _find_template_source(
+        self,
+        template_name: str,
+        config_path: Path,
+        project_root: Path = Path("."),
+        config: Optional[FlexlateConfig] = None,
+    ) -> Tuple[int, TemplateSource]:
+        """
+
+        :param template_name:
+        :param config_path:
+        :param project_root: The root of the current working project (may be a temp directory)
+        :param out_root:
+        :param orig_project_root: The root of the user's project (always stays the same, even
+            when working in a temp directory)
+        :return: index, applied template config tuple
+        """
+        config = config or self.load_config(project_root=project_root)
+        child_config = _get_or_create_child_config_by_path(config, config_path)
+        template_index: Optional[int] = None
+        for i, template_source in enumerate(child_config.template_sources):
+            if template_source.name == template_name:
+                template_index = i
+                break
+        if template_index is None:
+            raise CannotRemoveAppliedTemplateException(
+                f"Cannot find any template source with name {template_name}"
+            )
+        template_source = child_config.template_sources[template_index]
+        return template_index, template_source
 
     def get_num_applied_templates_in_child_config(
         self, child_config_path: Path, project_root: Path = Path(".")
@@ -389,6 +456,124 @@ class ConfigManager:
             seen_sources.add(source.name)
         return list(sources_with_templates.values())
 
+    def move_applied_template(
+        self,
+        template_name: str,
+        config_path: Path,
+        new_config_path: Path,
+        project_root: Path = Path("."),
+        out_root: Path = Path("."),
+        orig_project_root: Path = Path("."),
+    ):
+        config = self.load_config(project_root=project_root, adjust_applied_paths=False)
+        child_config = _get_or_create_child_config_by_path(config, config_path)
+        template_index, _ = self._find_applied_template(
+            template_name,
+            config_path,
+            project_root=project_root,
+            out_root=out_root,
+            orig_project_root=orig_project_root,
+            config=config,
+            adjust_applied_paths=False,
+        )
+        applied_template = child_config.applied_templates.pop(template_index)
+        new_child_config = _get_or_create_child_config_by_path(config, new_config_path)
+        new_child_config.applied_templates.append(applied_template)
+        self.save_config(config)
+
+    def move_template_source(
+        self,
+        template_name: str,
+        config_path: Path,
+        new_config_path: Path,
+        project_root: Path = Path("."),
+    ):
+        config = self.load_config(project_root=project_root, adjust_applied_paths=False)
+        child_config = _get_or_create_child_config_by_path(config, config_path)
+        template_index, _ = self._find_template_source(
+            template_name,
+            config_path,
+            project_root=project_root,
+            config=config,
+        )
+        template_source = child_config.template_sources.pop(template_index)
+        new_child_config = _get_or_create_child_config_by_path(config, new_config_path)
+        new_child_config.template_sources.append(template_source)
+        self.save_config(config)
+
+    def _get_applied_templates_and_sources_with_local_add_mode(
+        self,
+        project_root: Path = Path("."),
+        config: Optional[FlexlateConfig] = None,
+    ) -> List[AppliedTemplateWithSource]:
+        config = config or self.load_config(project_root=project_root)
+
+        return [
+            atws
+            for atws in self.get_applied_templates_with_sources(
+                project_root=project_root, config=config
+            )
+            if atws.applied_template.add_mode == AddMode.LOCAL
+        ]
+
+    def move_local_applied_templates_if_necessary(
+        self,
+        project_root: Path = Path("."),
+        orig_project_root: Path = Path("."),
+        renderer: MultiRenderer = MultiRenderer(),
+    ):
+        config = self.load_config(project_root=project_root)
+        applied_templates_with_sources = (
+            self._get_applied_templates_and_sources_with_local_add_mode(
+                project_root=project_root, config=config
+            )
+        )
+        for atwc in applied_templates_with_sources:
+            source = atwc.source
+            if source.is_local_template:
+                # Move source back to orig project so that relative template
+                # paths can be resolved
+                source.path = str(
+                    location_relative_to_new_parent(
+                        Path(source.path), project_root, orig_project_root, project_root
+                    ).resolve()
+                )
+            template = source.to_template()
+            if template.render_relative_root_in_output == Path("."):
+                # Should not need to move as config will not be in a subdirectory
+                continue
+            renderable = Renderable.from_applied_template_with_source(atwc)
+            new_relative_out_root = Path(
+                renderer.render_string(
+                    str(template.render_relative_root_in_output), renderable
+                )
+            )
+            if template.render_relative_root_in_output == new_relative_out_root:
+                # No need to move, render relative root was not a templated path
+                continue
+
+            orig_config_path = atwc.applied_template._config_file_location
+
+            render_root = (
+                orig_config_path.parent / atwc.applied_template._orig_root
+            ).resolve()
+            new_config_path = render_root / new_relative_out_root / "flexlate.json"
+            if orig_config_path == new_config_path:
+                # No need to move, still in the same location
+                continue
+
+            # Must have different location now, move it
+            # TODO: more efficient algorithm for updating locations of applied templates
+            #  Currently it needs to find the template twice
+            self.move_applied_template(
+                atwc.source.name,
+                orig_config_path,
+                new_config_path,
+                project_root=project_root,
+                out_root=atwc.applied_template._orig_root,
+                orig_project_root=orig_project_root,
+            )
+
 
 def _get_child_config_by_path(config: FlexlateConfig, path: Path) -> FlexlateConfig:
     for child_config in config.child_configs:
