@@ -19,7 +19,9 @@ class Template(abc.ABC):
         version: Optional[str] = None,
         target_version: Optional[str] = None,
         git_url: Optional[str] = None,
-        template_source_path: Optional[Path] = None,
+        template_source_path: Optional[str] = None,
+        render_relative_root_in_output: Path = Path("."),
+        render_relative_root_in_template: Path = Path("."),
     ):
         self.config = config
         self.path = path
@@ -28,6 +30,8 @@ class Template(abc.ABC):
         self.name = name or self.default_name
         self.version = version or self.folder_hash
         self.template_source_path = template_source_path or path
+        self.render_relative_root_in_output = render_relative_root_in_output
+        self.render_relative_root_in_template = render_relative_root_in_template
 
     @property
     def default_name(self) -> str:
