@@ -3,7 +3,9 @@ from pathlib import Path
 from typing import Sequence, Optional, List, Dict, Any
 
 from git import Repo
+from rich.prompt import Confirm
 
+from flexlate.cli_utils import confirm_user
 from flexlate.config_manager import ConfigManager
 from flexlate.constants import DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME
 from flexlate.finder.multi import MultiFinder
@@ -14,6 +16,13 @@ from flexlate.path_ops import (
 )
 from flexlate.render.multi import MultiRenderer
 from flexlate.render.renderable import Renderable
+from flexlate.styles import (
+    print_styled,
+    ACTION_REQUIRED_STYLE,
+    styled,
+    QUESTION_STYLE,
+    ALERT_STYLE,
+)
 from flexlate.template.base import Template
 from flexlate.ext_git import (
     stage_and_commit_all,
@@ -23,6 +32,9 @@ from flexlate.ext_git import (
     assert_repo_is_in_clean_state,
     temp_repo_that_pushes_to_branch,
     fast_forward_branch_without_checkout,
+    abort_merge,
+    reset_branch_to_commit_without_checkout,
+    get_branch_sha,
 )
 from flexlate.template_data import TemplateData, merge_data
 from flexlate.transactions.transaction import (
@@ -55,6 +67,11 @@ class Updater:
         project_root = Path(repo.working_dir)
         current_branch = repo.active_branch
 
+        # Save the status of the flexlate branches. We may need to roll back to this state
+        # if the user aborts the merge
+        merged_branch_sha = get_branch_sha(repo, merged_branch_name)
+        template_branch_sha = get_branch_sha(repo, template_branch_name)
+
         # Prepare the template branch, this is the branch that stores only the template files
         # Create it from the initial commit if it does not exist
         cwd = Path(os.getcwd())
@@ -103,6 +120,15 @@ class Updater:
             # paths back to how they were originally (relative if needed), so that there will not be
             # unexpected changes from relative to absolute paths in the user configs
             config_manager.update_templates(new_updates, project_root=temp_project_root)
+
+            # For applied templates with local add mode, with corresponding template sources that
+            # have a render_relative_root_in_output, may need to move the config
+            config_manager.move_local_applied_templates_if_necessary(
+                project_root=temp_project_root,
+                orig_project_root=project_root,
+                renderer=renderer,
+            )
+
             commit_message = create_transaction_commit_message(
                 _commit_message(renderables), transaction
             )
@@ -118,31 +144,57 @@ class Updater:
         checkout_template_branch(repo, merged_branch_name)
         merge_branch_into_current(repo, template_branch_name)
 
-        if not repo_has_merge_conflicts(repo):
-            # No conflicts, merge back into current branch
-            current_branch.checkout()
-            merge_branch_into_current(repo, merged_branch_name)
+        if repo_has_merge_conflicts(repo):
+            if no_input:
+                # Not receiving input, so there is no way the user could resolve conflicts
+                # during the process
+                print_styled("Repo has merge conflicts after update", ALERT_STYLE)
+                return
 
-            # TODO: Make CLI have a pause and resume capability
-            #  We should be running these post actions below afterwards regardless of whether
-            #  there was a merge conflict.
-
-            # Current working directory or out root may have been deleted if it was a remove operation
-            # and there was nothing else in the folder (git does not save folders without files)
-            ensure_exists_folders = [cwd]
-            for renderable in orig_renderables:
-                ensure_exists_folders.append(
-                    make_absolute_path_from_possibly_relative_to_another_path(
-                        renderable.out_root, project_root
-                    )
+            # Need to wait for user to resolve merge conflicts
+            print_styled(
+                "Repo has merge conflicts after update, please resolve them",
+                ACTION_REQUIRED_STYLE,
+            )
+            user_fixed = confirm_user(
+                styled("Conflicts fixed? n to abort", QUESTION_STYLE)
+            )
+            if not user_fixed:
+                print_styled(
+                    "Aborting update.",
+                    ALERT_STYLE,
                 )
-            for folder in ensure_exists_folders:
-                if not os.path.exists(folder):
-                    os.makedirs(folder)
+                # Abort merge and reset flexlate branches to original state
+                abort_merge(repo)
+                current_branch.checkout()
+                reset_branch_to_commit_without_checkout(
+                    repo, merged_branch_name, merged_branch_sha
+                )
+                reset_branch_to_commit_without_checkout(
+                    repo, template_branch_name, template_branch_sha
+                )
+                return
 
-            # Folder may have been deleted again while switching branches, so
-            # need to set cwd again
-            os.chdir(cwd)
+        # No conflicts, merge back into current branch
+        current_branch.checkout()
+        merge_branch_into_current(repo, merged_branch_name)
+
+        # Current working directory or out root may have been deleted if it was a remove operation
+        # and there was nothing else in the folder (git does not save folders without files)
+        ensure_exists_folders = [cwd]
+        for renderable in orig_renderables:
+            ensure_exists_folders.append(
+                make_absolute_path_from_possibly_relative_to_another_path(
+                    renderable.out_root, project_root
+                )
+            )
+        for folder in ensure_exists_folders:
+            if not os.path.exists(folder):
+                os.makedirs(folder)
+
+        # Folder may have been deleted again while switching branches, so
+        # need to set cwd again
+        os.chdir(cwd)
 
     def get_updates_for_templates(
         self,
