@@ -23,6 +23,7 @@ from tests.config import (
     COOKIECUTTER_ONE_MODIFIED_VERSION,
     COPIER_ONE_VERSION,
     COPIER_ONE_MODIFIED_VERSION,
+    COPIER_OUTPUT_SUBDIR_DIR,
 )
 
 
@@ -79,6 +80,12 @@ def copier_one_template() -> CookiecutterTemplate:
     yield finder.find(str(COPIER_ONE_DIR), COPIER_ONE_DIR)
 
 
+@pytest.fixture
+def copier_output_subdir_template() -> CookiecutterTemplate:
+    finder = CopierFinder()
+    yield finder.find(str(COPIER_OUTPUT_SUBDIR_DIR), COPIER_OUTPUT_SUBDIR_DIR)
+
+
 @pytest.fixture
 def cookiecutter_two_template() -> CookiecutterTemplate:
     finder = CookiecutterFinder()
@@ -93,7 +100,29 @@ def cookiecutter_remote_template() -> CookiecutterTemplate:
         local_path, name = get_local_repo_path_and_name_cloning_if_repo_url(
             COOKIECUTTER_REMOTE_URL, dst_folder=temp_path
         )
-        yield finder.find(COOKIECUTTER_REMOTE_URL, local_path, name=name)
+        template = finder.find(COOKIECUTTER_REMOTE_URL, local_path, name=name)
+        # This would normally happen when loading template from source in config
+        template.template_source_path = COOKIECUTTER_REMOTE_URL
+        yield template
+
+
+@pytest.fixture()
+def cookiecutter_remote_version_one_template() -> CookiecutterTemplate:
+    finder = CookiecutterFinder()
+    with tempfile.TemporaryDirectory() as temp_dir:
+        temp_path = Path(temp_dir)
+        local_path, name = get_local_repo_path_and_name_cloning_if_repo_url(
+            COOKIECUTTER_REMOTE_URL, COOKIECUTTER_REMOTE_VERSION_1, dst_folder=temp_path
+        )
+        template = finder.find(
+            COOKIECUTTER_REMOTE_URL,
+            local_path,
+            name=name,
+            version=COOKIECUTTER_REMOTE_VERSION_1,
+        )
+        # This would normally happen when loading template from source in config
+        template.template_source_path = COOKIECUTTER_REMOTE_URL
+        yield template
 
 
 @pytest.fixture
