@@ -5,16 +5,19 @@ from typing import Optional, Sequence
 from unittest.mock import patch
 
 import pytest
+from _pytest.monkeypatch import MonkeyPatch
 from git import Repo, Head
 
 from flexlate.config import FlexlateConfig, TemplateSource, TemplateSourceWithTemplates
 from flexlate.exc import GitRepoDirtyException
 from flexlate.finder.multi import MultiFinder
+from flexlate.render.specific import cookiecutter
 from flexlate.template.base import Template
 from flexlate.template.cookiecutter import CookiecutterTemplate
-from flexlate.constants import DEFAULT_MERGED_BRANCH_NAME
+from flexlate.constants import DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME
 from flexlate.template.types import TemplateType
 from flexlate.transactions.transaction import FlexlateTransaction
+from flexlate.update import main
 from flexlate.update.main import Updater
 from flexlate.update.template import TemplateUpdate
 from tests.config import (
@@ -90,6 +93,36 @@ def test_update_modify_template(
     )
 
 
+def test_update_modify_data(
+    cookiecutter_one_template: CookiecutterTemplate,
+    repo_with_gitignore_and_template_branch_from_cookiecutter_one: Repo,
+    update_transaction: FlexlateTransaction,
+):
+    repo = repo_with_gitignore_and_template_branch_from_cookiecutter_one
+    updater = Updater()
+    template_updates = updater.get_updates_for_templates(
+        [cookiecutter_one_template],
+        data=[{"a": "d", "c": "e"}],
+        project_root=GENERATED_REPO_DIR,
+    )
+    updater.update(repo, template_updates, update_transaction, no_input=True)
+    assert (
+        cookiecutter_one_generated_text_content(folder="d", gen_dir=GENERATED_REPO_DIR)
+        == "de"
+    )
+    assert (GENERATED_REPO_DIR / "ignored" / "ignored.txt").exists()
+    assert (GENERATED_REPO_DIR / ".gitignore").exists()
+
+    # Original b directory should no longer exist, everything moved to d
+    assert not (GENERATED_REPO_DIR / "b").exists()
+
+    config_path = GENERATED_REPO_DIR / "d" / "flexlate.json"
+    config = FlexlateConfig.load(config_path)
+    assert len(config.applied_templates) == 1
+    at = config.applied_templates[0]
+    assert at.data == {"a": "d", "c": "e"}
+
+
 def test_update_modify_template_conflict(
     cookiecutter_one_modified_template: CookiecutterTemplate,
     repo_from_cookiecutter_one_with_modifications: Repo,
@@ -104,6 +137,68 @@ def test_update_modify_template_conflict(
     assert repo_has_merge_conflicts(repo)
 
 
+@patch.object(
+    cookiecutter, "prompt_for_config", lambda context, no_input: {"a": "b", "c": ""}
+)
+def test_update_modify_template_conflict_with_resolution(
+    cookiecutter_one_modified_template: CookiecutterTemplate,
+    repo_from_cookiecutter_one_with_modifications: Repo,
+    update_transaction: FlexlateTransaction,
+):
+    repo = repo_from_cookiecutter_one_with_modifications
+    manual_commit_message = "Manually resolve conflicts"
+
+    def _resolve_conflicts_then_type_yes(prompt: str) -> bool:
+        stage_and_commit_all(repo, manual_commit_message)
+        return True
+
+    updater = Updater()
+    template_updates = updater.get_updates_for_templates(
+        [cookiecutter_one_modified_template], project_root=GENERATED_FILES_DIR
+    )
+    with patch.object(main, "confirm_user", _resolve_conflicts_then_type_yes):
+        updater.update(repo, template_updates, update_transaction)
+    assert not repo_has_merge_conflicts(repo)
+
+    for branch_name in [
+        "master",
+        DEFAULT_MERGED_BRANCH_NAME,
+    ]:
+        branch: Head = repo.branches[branch_name]  # type: ignore
+        branch.checkout()
+        assert repo.commit().message == manual_commit_message + "\n"
+
+
+@patch.object(
+    cookiecutter, "prompt_for_config", lambda context, no_input: {"a": "b", "c": ""}
+)
+def test_update_modify_template_conflict_with_reject(
+    cookiecutter_one_modified_template: CookiecutterTemplate,
+    repo_from_cookiecutter_one_with_modifications: Repo,
+    update_transaction: FlexlateTransaction,
+):
+    repo = repo_from_cookiecutter_one_with_modifications
+
+    def _reject_update(prompt: str) -> bool:
+        return False
+
+    updater = Updater()
+    template_updates = updater.get_updates_for_templates(
+        [cookiecutter_one_modified_template], project_root=GENERATED_FILES_DIR
+    )
+    with patch.object(main, "confirm_user", _reject_update):
+        updater.update(repo, template_updates, update_transaction)
+
+    assert repo.commit().message == "Prepend cookiecutter text with hello\n"
+
+    for branch_name in [DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME]:
+        branch: Head = repo.branches[branch_name]  # type: ignore
+        branch.checkout()
+        assert_main_commit_message_matches(
+            repo.commit().message, "Update flexlate templates"
+        )
+
+
 @pytest.mark.parametrize(
     "remote_target_version", [None, "main", COOKIECUTTER_REMOTE_VERSION_2]
 )
